@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 """
-Copyright (c) 2006-2018 sqlmap developers (http://sqlmap.org/)
+Copyright (c) 2006-2020 sqlmap developers (http://sqlmap.org/)
 See the file 'LICENSE' for copying permission
 """
 
@@ -10,9 +10,6 @@ from lib.core.exception import SqlmapUnsupportedFeatureException
 from plugins.generic.enumeration import Enumeration as GenericEnumeration
 
 class Enumeration(GenericEnumeration):
-    def __init__(self):
-        GenericEnumeration.__init__(self)
-
     def getCurrentUser(self):
         warnMsg = "on SQLite it is not possible to enumerate the current user"
         logger.warn(warnMsg)
@@ -21,10 +18,12 @@ class Enumeration(GenericEnumeration):
         warnMsg = "on SQLite it is not possible to get name of the current database"
         logger.warn(warnMsg)
 
-    def isDba(self):
+    def isDba(self, user=None):
         warnMsg = "on SQLite the current user has all privileges"
         logger.warn(warnMsg)
 
+        return True
+
     def getUsers(self):
         warnMsg = "on SQLite it is not possible to enumerate the users"
         logger.warn(warnMsg)
@@ -37,7 +36,7 @@ class Enumeration(GenericEnumeration):
 
         return {}
 
-    def getPrivileges(self, *args):
+    def getPrivileges(self, *args, **kwargs):
         warnMsg = "on SQLite it is not possible to enumerate the user privileges"
         logger.warn(warnMsg)
 
@@ -62,3 +61,9 @@ class Enumeration(GenericEnumeration):
     def getHostname(self):
         warnMsg = "on SQLite it is not possible to enumerate the hostname"
         logger.warn(warnMsg)
+
+    def getStatements(self):
+        warnMsg = "on SQLite it is not possible to enumerate the SQL statements"
+        logger.warn(warnMsg)
+
+        return []
